/*
 * Copyright (c) Members of the EGEE Collaboration. 2009-2010.
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sd.h"

int main(int argc, char** argv)
{
   int optind = 1;
   const char* svcFilter = "";
   const char* voFilter = "";
   const char* dataFilter = "";
   const char* serverUrl = "";

   unsigned int haveAuthzFilter = 0;

   //Loop through all our command line arguments*/
   while ( (optind < argc) && (argv[optind][0] == '-') )
   {
      const char* sw = argv[optind];

      if ( strcmp(sw, "-sf") == 0 )
      {
         optind++;
         svcFilter = argv[optind];
         //cout << "Service Filter: " << svcFilter << endl;
      }

      else if ( strcmp(sw, "-vf") == 0 )
      {
         optind++;
         voFilter = argv[optind];
         haveAuthzFilter = 1;
         //cout << "VO Filter: " << voFilter << endl;
      }

      else if ( strcmp(sw, "-df") == 0 )
      {
         optind++;

         dataFilter = argv[optind];
      }

      else if ( strcmp(sw, "-url") == 0 )
      {
         optind++;

         serverUrl = argv[optind];
      }

      else if ( strcmp(sw, "-h") == 0 ) 
      {
         (void)printf("%s :\n", argv[0]);
         (void)printf("\n");
         (void)printf("Allowable arguments:\n");
         (void)printf("\t-sf <service filter>\n");
         (void)printf("\t-vf <authz filter>\n");
         (void)printf("\t-df <data filter>\n");
         (void)printf("\t-url <Server URL>\n");
         (void)printf("\t-h Print this message\n");
         (void)printf("\n");
         return 0;
      }

      else
      {
         (void)printf("Unknown switch: %s\n", argv[optind]);
         (void)printf("Run with -h for help\n");
         exit(0);
      }
      optind++;
   }

   DISCOVERER d;
   SD_RETURN_TYPE ret = SD_OK;;

   char* error;

   ret = InitDiscoverer(&d, serverUrl, &error);

   if ( ret == SD_FAIL )
   {
      (void)printf("InitDiscoverer() failed - %s\n", error);
      free(error);
   }

   SERVICE_DESCRIPTION_ARRAY arr;
   if ( haveAuthzFilter != 0 )
   {
      ret = ListServices3(&d, svcFilter, voFilter, dataFilter, &arr, &error);
   }

   else
   {
      ret = ListServices2(&d, svcFilter, dataFilter, &arr, &error);
   }

   if ( ret == SD_FAIL )
   {
      (void)printf("ListServices3() failed - %s\n", error);
      free(error);
   }

   else
   {
      unsigned int count;
      (void)printf("List services returned %u results\n", arr.count);

      /*Loop over each service*/
      for ( count = 0; count < arr.count; ++count )
      {
         ATTRIBUTE_NAMES attArr;

         (void)printf("SERVICE #%u\n", count);
         (void)printf("-----------------------------------------------\n");

         SERVICE_DESCRIPTION sd = arr.descriptionArray[count];

         if ( ServiceDescriptionListAttributes(&sd, &attArr, NULL) == SD_OK )
         {
            char* url = ServiceDescriptionGetUrl(&sd, NULL);

            if ( url != NULL )
            {
               (void)printf("get_url() : %s\n",
                            url);

               free(url);
            }

            //Get related services
            /*SERVICE_DESCRIPTION_ARRAY relatedArr;
            if ( ServiceDescriptionGetRelatedServices(&sd,
                                                      &relatedArr,
                                                      NULL) == SD_OK )
            {
               (void)printf("Related Services : %u\n", relatedArr.count);
               ReleaseServiceDescriptionArray(&relatedArr, NULL);
            }*/

            unsigned int attCount;

            /*Loop over each attribute*/
            for ( attCount = 0; attCount < attArr.count; ++attCount )
            {
               ATTRIBUTE_VALUES attVals;
               const char* att = attArr.attributeNames[attCount];

               if ( ServiceDescriptionGetAttribute(&sd,
                                                   att,
                                                   &attVals,
                                                   NULL) == SD_OK )
               {
                  unsigned int valCount;

                  /*Loop over each value for this attribute*/
                  for ( valCount = 0; valCount < attVals.count; ++valCount )
                  {
                     (void)printf("%s : %s\n",
                                  att,
                                  attVals.attributeValues[valCount]);
                  }
               }
               ReleaseAttributeValues(&attVals, NULL);
            }
            ReleaseAttributeNames(&attArr, NULL);

            /*Get the SERVICE_DATA for this service*/
            SERVICE_DATA* data = ServiceDescriptionGetData(&sd, NULL);

            if ( data != NULL )
            {
               ATTRIBUTE_NAMES dataArr;

               if ( ServiceDataListAttributes(data,
                                              &dataArr,
                                              NULL) == SD_OK )
               {
                  unsigned int dataCount;

                  if ( dataArr.count > 0 )
                  {
                     (void)printf("------------------------"
                                  "-----------------------\n");
                     (void)printf("DATA\n");
                     (void)printf("------------------------"
                                  "-----------------------\n");
                  }

                  /*Loop over each data attribute*/
                  for ( dataCount = 0; dataCount < dataArr.count; ++dataCount )
                  {
                     ATTRIBUTE_VALUES dataVals;
                     const char* att = dataArr.attributeNames[dataCount];

                     if ( ServiceDataGetAttribute(data,
                                                  att,
                                                  &dataVals,
                                                  NULL) == SD_OK )
                     {
                        unsigned int valCount;

                        /*Loop over each value for this data attribute*/
                        for ( valCount = 0;
                              valCount < dataVals.count;
                              ++valCount )
                        {
                           (void)printf("%s : %s\n",
                                        att,
                                        dataVals.attributeValues[valCount]);
                        }
                     }
                     ReleaseAttributeValues(&dataVals, NULL);
                  }
                  ReleaseAttributeNames(&dataArr, NULL);
               }
               /*Release the SERVICE_DATA structure then free it*/
               ReleaseServiceData(data, NULL);
               free(data);
            }

            (void)printf("-----------------------------------------------\n");
         }
      }
      /*Free the memory from our array of SERVICE_DESCRIPTION structs*/
      ret = ReleaseServiceDescriptionArray(&arr, &error);

      if ( ret == SD_FAIL )
      {
         (void)printf("ReleaseServiceDescriptionArray() failed - %s\n",
                      error);
         free(error);
      }
   }

   ret = ReleaseDiscoverer(&d, &error);

   if ( ret == SD_FAIL )
   {
      (void)printf("ReleaseDiscoverer() failed - %s\n", error);
      free(error);
   }
   return 0;
}
