/*
 * Copyright (c) Members of the EGEE Collaboration. 2009-2010.
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
%module Discoverer
%{
   #include "discoverer.hpp"
   #include "session.hpp"
   #include "exception.hpp"
%}

%include "exception.i"
%include "std_string.i"
%include "std_vector.i"

/*service_description doesn't have a default constructor*/
%std_nodefconst_type(saga::sd::service_description);

%rename list_attributes listAttributes;
%rename find_attributes findAttributes;
%rename remove_attributes removeAttributes;

%rename get_attribute getAttribute;
%rename get_vector_attribute getVectorAttribute;

%rename attribute_exists attributeExists;
%rename attribute_is_readonly attributeIsReadOnly;
%rename attribute_is_writable attributeIsWritable;
%rename attribute_is_vector attributeIsVector;
%rename attribute_is_removable attributeIsRemovable;

/*Hopefully this will translate C++ exceptions into Python ones*/
%exception
{
   try
   {
      $action
   }

   catch ( const saga::exception& ex )
   {
      PyErr_SetString(PyExc_RuntimeError, ex.get_message());
      return NULL;
   }
}

namespace std
{
   %template(vectorsd) vector<saga::sd::service_description>;

   %template(vectorstr) vector<std::string>;
}

namespace saga
{
   %rename url Url;
   class url
   {
      public:
         url(saga::url const& rhs);
         url(std::string const& urlstr);
   };

   class session
   {
      public:
         session(bool default_session = true);
         ~session();
   };
};

namespace saga
{
   namespace sd
   {

      %rename discoverer Discoverer;
      class discoverer
      {
         public:
         discoverer(session const& s, saga::url loc = saga::url());
         discoverer(saga::url loc = saga::url());
         ~discoverer();

         %rename list_services listServices;
         std::vector<saga::sd::service_description>
            list_services(std::string svc,
                          std::string data);

         std::vector<saga::sd::service_description>
            list_services(std::string svc,
                          std::string vo,
                          std::string data);
      };

      %rename service_description ServiceDescription;
      class service_description
      {
         public:
         service_description(saga::url loc);
         ~service_description();

         std::vector<std::string> list_attributes();

         bool attribute_is_vector(std::string);

         std::string get_attribute(std::string key) const;
         std::vector<std::string> get_vector_attribute(std::string key) const;

         %rename get_url getUrl;
         std::string get_url();

         %rename get_relates_services getRelatedServices;
         std::vector<saga::sd::service_description> get_related_services();

         %rename get_data getData;
         saga::sd::service_data& get_data();
      };

      %rename service_data ServiceData;
      class service_data
      {
         public:
         service_data();
         ~service_data();

         std::string get_attribute(std::string key) const;
         std::vector<std::string> get_vector_attribute(std::string key) const;

         std::vector<std::string> list_attributes() const;
         std::vector<std::string> find_attributes(std::string pattern) const;

         bool attribute_exists(std::string key) const;
         bool attribute_is_readonly(std::string key) const;
         bool attribute_is_writable(std::string key) const;
         bool attribute_is_vector(std::string key) const;
         bool attribute_is_removable(std::string key) const;
      };
   };

   namespace detail
   {
      template <typename Derived>
      struct attribute
      {
         public:

         std::string get_attribute(std::string key) const;
         void set_attribute(std::string key,
                            std::string value);

         std::vector<std::string> get_vector_attribute(std::string key) const;
         void set_vector_attribute(std::string key,
                                  std::vector<std::string> value);

         void remove_attribute(std::string key);

         std::vector<std::string> list_attributes() const;
         std::vector<std::string> find_attributes(std::string pattern) const;

         bool attribute_exists(std::string key) const;
         bool attribute_is_readonly(std::string key) const;
         bool attribute_is_writable(std::string key) const;
         bool attribute_is_vector(std::string key) const;
         bool attribute_is_removable(std::string key) const;
      };
   };
};

