#!/usr/bin/env python
#
# Copyright (c) Members of the EGEE Collaboration. 2009-2010.
# See http://www.eu-egee.org/partners/ for details on the copyright
# holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from Discoverer import *
from optparse import OptionParser
import sys

def main():

   #Set up our options parser
   usage = "usage: %prog [options] arg"
   parser = OptionParser(usage)

   parser.add_option("--sf",
                     "-s",
                     action="store",
                     type="string",
                     dest="serviceFilter",
                     default="",
                     help="Services filter")

   #No default for Authz Filter
   #as we need to know if it was supplied or not
   parser.add_option("--af",
                     "-a",
                     action="store",
                     type="string",
                     dest="authzFilter",
                     help="Authz filter (optional)")

   parser.add_option("--df",
                     "-d",
                     action="store",
                     type="string",
                     dest="dataFilter",
                     default="",
                     help="Data filter")

   parser.add_option("--url",
                     "-u",
                     action="store",
                     type="string",
                     dest="serverUrl",
                     default="",
                     help="URL of information server")

   (options, args) = parser.parse_args()

   #serverUrl will be set to either an URL or ""
   #in which case the adaptor will have to sort something out
   url = Url(options.serverUrl)

   disco = Discoverer(url)

   #Default empty tuple
   vec = ()

   #Depending on wheter or not we have an authzFilter
   #call the correct version of listServices
   if options.authzFilter == None:
      vec = disco.listServices(options.serviceFilter,
                               options.dataFilter)

   else:
      vec = disco.listServices(options.serviceFilter,
                               options.authzFilter,
                               options.dataFilter)

   #Our count used for the number of services
   count = 0

   if len(vec) == 0:
      print 'No data returned'

   for sd in vec:
      print "-----------------------------------------------"
      print "SERVICE #" ,  count
      print "-----------------------------------------------"

      names = sd.listAttributes()

      for name in names:
         if sd.attributeIsVector(name):
            values = sd.getVectorAttribute(name)

            #Output the attribute value pairs
            for value in values:
               print name, " : ", value

         else:
            #Output the attribute value pair
            value = sd.getAttribute(name)
            print name, " : ", value

      #We've finished with the standard attributes
      #Now look for ServiceData attributes
      #Get the ServiceData
      data = ()
      try:
         data = sd.getData();

      except Exception, e:
         #Not a lot we can do about this
         #So carry on regardless,
         #ignoring the service_data from this service
         continue

      dataNames = data.listAttributes();

      #Do we have any data attributes?
      if len(dataNames) > 0:
         print "-----------------------------------------------"
         print "SERVICE DATA"
         print "-----------------------------------------------"

      #Loop through al the data
      for dataName in dataNames:
         if data.attributeIsVector(dataName):
            dataValues = data.getVectorAttribute(dataName);

            for dataValue in dataValues:
               print dataName, " : ", dataValue

         else:
            dataValue = data.getAttribute(dataName);
            print dataName, " : ", dataValue

      print

      #Finished with this service
      #so increment our count and continue
      count = count + 1

if __name__ == "__main__" :
   main()

