#!/usr/bin/perl -w
#
# Copyright (c) Members of the EGEE Collaboration. 2009-2010.
# See http://www.eu-egee.org/partners/ for details on the copyright
# holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
use strict;

my $errors = 0;
my $tests = 0;

#Function prototypes
sub GetCountForService($$);
sub GetCountForNotServiceType($);
sub GetCountForServices(@);
sub GetCountForVo($);
sub GetCountForVoIn(@);
sub ConfirmValueNotPresentForVo($@);
sub PassFail($$);

my $app = 'saga-sd --da --sa';

#Due to the automatic Authz filter generation
#we need to provide an empty Authz filter in some cases
my $appNoAF = "$app --af \"\"";

#Variables to hold common search strings
my $searchType = '^Service Type';
my $searchService = '^Url';

#General tests
my @outArr = qx($appNoAF --sf "" | grep '$searchService');
my $totalNumServices = @outArr;

if ( $totalNumServices == 0 )
{
   die "ERROR: No services returned.  Exiting...\n";
}

print "Total Number of Services = $totalNumServices\n\n";

######################################################################
#                       Service Filter tests                         #
######################################################################
#Now more specifics
my $rbType = 'ResourceBroker';
my $numRbServices = GetCountForService("Type", $rbType);

#Now look for another service type
print "\n";
my $srmType = 'SRM';
my $numSrmServices = GetCountForService("Type", $srmType);

#Now look for a service Name
print "\n";
my $serviceName = 'IN2P3-LAPP-rb';
my $numNamedService = GetCountForService("Name", $serviceName);

#Now look for a service Url
print "\n";
my $serviceUrl = 'ldap://site-bdii.gridpp.rl.ac.uk:2170/mds-vo-name=RAL-LCG2,o=grid';
my $numUrlService = GetCountForService("Url", $serviceUrl);

#Now look for a service Uid
print "\n";
my $serviceUid = 'httpg://se.grid.vgtu.lt:8443/srm/managerv1';
my $numUidService = GetCountForService("Uid", $serviceUid);

#Check that these two add up
print "\n";
print "Check that OR works in service filtes\n";

my $numServices = GetCountForServices(('Type', $rbType,
                                       'Type', $srmType));
$tests++;
my $expectedTotal = $numRbServices + $numSrmServices;
PassFail($expectedTotal, $numServices);

#Check that if we ask for the same thing twice we only get it once
print "\n";
print "Check OR works correctly when the same thing is asked for twice\n";
$numServices = GetCountForServices(('Type', $rbType,
                                    'Type', $rbType));
$tests++;
PassFail($numRbServices, $numServices);

print "\n";
print "Check AND works correctly when the same thing is asked for twice\n";
@outArr = qx($appNoAF --sf "TYPE = 'ResourceBroker' AND
                             TYPE = 'ResourceBroker'" --sa);
$numServices = grep(/$searchType/i, @outArr);
$tests++;
PassFail($numRbServices, $numServices);

#Check that NOT works
print "\n";
print "Check that NOT works for Service Type\n";
my $numNotRbServices = GetCountForNotServiceType($rbType);
$tests++;
PassFail($totalNumServices - $numRbServices, $numNotRbServices);

#Check that NOT and = work
print "\n";
print "Check that 'x NOT =' OR 'y =' works for Service Type\n";
print "Check TYPE = ResourceBroker OR Type != Resource Broker\n";
@outArr = qx($appNoAF --sf "TYPE != '$rbType' OR TYPE = '$rbType'");
$numServices = grep(/$searchType/i, @outArr);
$tests++;
PassFail($numServices, $numServices);

#Check that NOT and = work
print "\n";
print "Check that 'x NOT =' AND 'y =' works for Service Type\n";
print "Check TYPE = ResourceBroker AND Type != Resource Broker\n";
@outArr = qx($appNoAF --sf "TYPE != '$rbType' AND TYPE = '$rbType'"); 
$numServices = grep(/$searchType/i, @outArr);
$tests++;
PassFail(0, $numServices);

#Check that LIKE works
print "\n";
print "Check that LIKE works for Service Type\n";
print "Check TYPE LIKE Res%ker\n";
@outArr = qx($appNoAF --sf "TYPE LIKE 'Res%ker'"); 
$numServices = grep(/Res.*ker/, grep(/$searchType/i, @outArr));
$tests++;
PassFail($numRbServices, $numServices);

#Check that NOT LIKE works
print "\n";
print "Check that NOT LIKE works for Service Type\n";
print "Check NOT TYPE LIKE Res%ker\n";
@outArr = qx($appNoAF --sf "NOT TYPE LIKE 'Res%ker'"); 
$numServices = grep(!/Res.*ker/, grep(/$searchType/i, @outArr));
$tests++;
PassFail($totalNumServices - $numRbServices, $numServices);

#Check that AND works
print "\n";
print "Check that AND works\n";
print "Check TYPE = ResourceBroker AND NAME = IN2P3-LAPP-rb\n";
my $totalNumRbNameServices = GetCountForServices(('Type', $rbType,
                                                  'Name', 'IN2P3-LAPP-rb'));
@outArr = qx($appNoAF --sf "TYPE = 'ResourceBroker' AND
                             NAME = 'IN2P3-LAPP-rb'"); 
my $totalNumAndServices = grep(/$searchType/i, @outArr);
$tests++;

PassFail($numRbServices + $numNamedService - $totalNumRbNameServices,
         $totalNumAndServices);

#Check that AND and NOT works
print "\n";
print "Check that AND and NOT works\n";
print "Check TYPE <> ResourceBroker AND NAME = IN2P3-LAPP-rb\n";

@outArr = qx($appNoAF --sf "TYPE <> 'ResourceBroker' AND
                             NAME = 'IN2P3-LAPP-rb'"); 
my $totalNumAndNotServices = grep(/$searchType/i, @outArr);
$tests++;
PassFail($numNamedService - $totalNumAndServices,
         $totalNumAndNotServices);

#Check that AND and NOT works
print "\n";
print "Check that AND and NOT works\n";
print "Check TYPE = ResourceBroker AND NAME <> IN2P3-LAPP-rb\n";

@outArr = qx($appNoAF --sf "TYPE = 'ResourceBroker' AND
                             NAME <> 'IN2P3-LAPP-rb'"); 
$totalNumAndNotServices = grep(/$searchType/i, @outArr);
$tests++;
PassFail($totalNumRbNameServices - $totalNumAndServices,
         $totalNumAndNotServices);

#Check that AND and NOT works
print "\n";
print "Check that AND and NOT works\n";
print "Check TYPE = ResourceBroker AND TYPE <> ResourceBroker\n";

@outArr = qx($appNoAF --sf "TYPE = 'ResourceBroker' AND
                             TYPE <> 'ResourceBroker'"); 
$numServices = grep(/$searchType/i, @outArr);
$tests++;
PassFail(0, $numServices);

#Check that AND and OR works
print "\n";
print "Check that AND and OR works\n";
print "Check TYPE = ResourceBroker OR TYPE <> ResourceBroker\n";

@outArr = qx($appNoAF --sf "TYPE = 'ResourceBroker' OR
                             TYPE <> 'ResourceBroker'"); 
$numServices = grep(/$searchType/i, @outArr);
$tests++;
PassFail($totalNumServices, $numServices);

######################################################################
#                          VO Filter tests                           #
######################################################################
#Get a few counts that we'll use later
print "\n";
print "Get all services with VO = 'atlas'\n";
my $numAtlas = GetCountForVo('atlas');

print "\n";
print "Get all services with VO = 'dteam'\n";
my $numDteam = GetCountForVo('dteam');

#There shouldn't be a VO = 'aaaa'
#except GLUE2 can specify VO = ALL
#so this might match
print "\n";
print "Get all services with VO = 'aaaa'\n";
my $numAaaa = GetCountForVo('aaaa');

print "\n";
print "Get all services with VO IN ('dteam', 'dteamby')\n";
my $numDteamStar = GetCountForVoIn(('dteam', 'dteamby'));

#Check OR returns the same as IN
print "\n";
print "Check that VO = 'atlas' OR VO = 'dteam' is the same as " .
      "VO IN ('atlas', 'dteam')\n";
@outArr = qx($app --af "VO = 'atlas' OR VO = 'dteam'");
my $totalOrVo = grep(/$searchService/, @outArr);

@outArr = qx($app --af "VO IN ('atlas', 'dteam')");
my $totalInVo = grep(/$searchService/, @outArr);
$tests++;
PassFail($totalOrVo, $totalInVo);

#Check '=' returns the same as 'not <>'
print "\n";
print "Check that VO = 'atlas' is the same as " .
      "NOT VO <> 'atlas'\n";
@outArr = qx($app --af "NOT VO <> 'atlas'");
my $totalNotNotAtlas = grep(/$searchService/, @outArr);
$tests++;
PassFail($numAtlas, $totalNotNotAtlas);

#Check 'not =' returns the same as '<>'
print "\n";
print "Check that NOT VO = 'atlas' is the same as " .
      "VO <> 'atlas'\n";

print "NOT VO = 'atlas'\n";
@outArr = qx($app --af "NOT VO = 'atlas'");
ConfirmValueNotPresentForVo('atlas', @outArr);
my $totalNotAtlas = grep(/$searchService/, @outArr);
$tests++;
PassFail($totalNumServices - $numAtlas, $totalNotAtlas);

print "\n";
print "VO <> 'atlas'\n";
@outArr = qx($app --af "VO <> 'atlas'");
$totalNotAtlas = grep(/$searchService/, @outArr);
$tests++;
PassFail($totalNumServices - $numAtlas, $totalNotAtlas);

#Check that if we ask for the same thing twice we only get it once
print "\n";
print "Check OR works correctly when the same thing is asked for twice\n";
@outArr = qx($app --af "VO = 'atlas' OR VO = 'atlas'");
$numServices = grep(/$searchService/, @outArr);
$tests++;
PassFail($numAtlas, $numServices);

print "\n";
print "Check AND works correctly when the same thing is asked for twice\n";
@outArr = qx($app --af "VO = 'atlas' AND VO = 'atlas'");
$numServices = grep(/$searchService/, @outArr);
$tests++;
PassFail($numAtlas, $numServices);

#Check AND and NOT
print "\n";
print "Check that VO = 'atlas' AND VO = 'dteam' is the same as " .
      "VO = 'atlas' OR VO = 'dteam' - " .
      "(VO = 'atlas' AND VO <> 'dteam' + VO = 'dteam' AND VO <> 'atlas')\n";
@outArr = qx($app --af "VO = 'atlas' AND VO = 'dteam'");
my $totalAndVo = grep(/$searchService/, @outArr);
print "$totalAndVo records for VO = 'atlas' and VO = 'dteam'\n";

print "$totalOrVo records for VO = 'dteam' OR VO <> 'atlas'\n";

@outArr = qx($app --af "VO = 'atlas' AND VO <> 'dteam'");
my $totalNot1 = grep(/$searchService/, @outArr);
print "$totalNot1 records for VO = 'atlas' and VO <> 'dteam'\n";
ConfirmValueNotPresentForVo('dteam', @outArr);

@outArr = qx($app --af "VO = 'dteam' AND VO <> 'atlas'");
my $totalNot2 = grep(/$searchService/, @outArr);
print "$totalNot2 records for VO = 'dteam' AND VO <> 'atlas'\n";
ConfirmValueNotPresentForVo('atlas', @outArr);
$tests++;
PassFail($totalAndVo, $totalOrVo - $totalNot1 - $totalNot2);

#Check NOT IN works
print "\n";
print "Check NOT VO IN ('atlas', 'dteam') works correctly\n";
@outArr = qx($app --af "NOT VO IN ('atlas', 'dteam')");
ConfirmValueNotPresentForVo('atlas', @outArr);
ConfirmValueNotPresentForVo('dteam', @outArr);
my $totalNotIn = grep(/$searchService/, @outArr);
$tests++;
PassFail($totalNumServices - $totalOrVo, $totalNotIn);

#Check that LIKE works
print "\n";
print "Check that LIKE works for VO\n";
print "Check VO LIKE dteam%\n";
@outArr = qx($app --af "VO LIKE 'dteam%'"); 
$numServices = grep(/$searchService/, @outArr);
$tests++;
PassFail($numDteamStar, $numServices);

#Check that NOT LIKE works
print "\n";
print "Check that NOT LIKE works for VO\n";
print "Check NOT VO LIKE dteam%\n";
@outArr = qx($app --af "NOT VO LIKE 'dteam%'"); 
$numServices = grep(/$searchService/, @outArr);
$tests++;
PassFail($totalNumServices - $numDteamStar, $numServices);

#Check parenthesis
print "\n";
print "Redo previous check using parenthesis\n";
print "(VO = 'atlas' AND VO <> 'dteam') OR (VO = 'dteam' and VO <> 'atlas')\n";
@outArr = qx($app --af "(VO = 'atlas' AND VO <> 'dteam') OR
                         (VO = 'dteam' AND VO <> 'atlas')");
$numServices = grep(/$searchService/, @outArr);
print "$numServices records for (VO = 'atlas' AND VO <> 'dteam') OR " .
                               "(VO = 'dteam' and VO <> 'atlas')\n";
$tests++;
PassFail($totalAndVo, $totalOrVo - $numServices);

#Check parenthesis
print "\n";
print "Check parenthesis\n";
print "VO = 'aaaa' AND (VO = 'atlas' OR VO = 'dteam')\n";
@outArr = qx($app --af "VO = 'aaaa' AND (VO = 'atlas' OR VO = 'dteam')");
$numServices = grep(/$searchService/, @outArr);
PassFail($numAaaa, $numServices);

#Check parenthesis
print "\n";
print "Check parenthesis\n";
print "(VO = 'aaaa' AND VO = 'atlas') OR VO = 'dteam'\n";
@outArr = qx($app --af "(VO = 'aaaa' AND VO = 'atlas') OR VO = 'dteam'");
$numServices = grep(/$searchService/, @outArr);
PassFail($numDteam, $numServices);

#Check parenthesis
print "\n";
print "Check parenthesis\n";
print "(VO = 'aaaa' AND VO = 'atlas' OR VO = 'dteam')\n";
@outArr = qx($app --af "(VO = 'aaaa' AND VO = 'atlas' OR VO = 'dteam')");
$numServices = grep(/$searchService/, @outArr);
PassFail($numDteam, $numServices);

print "\n";
print "Ran $tests tests and had $errors ";
if ( $errors == 1 ) {
    print "error\n\n";
} else {
    print "errors\n\n";
}
if ( $errors != 0 ) {
    exit 1;
}
######################################################################
#                          Helper Functions                          #
######################################################################
sub GetCountForService($$) {
   my $serviceAttr = shift;
   my $serviceValue = shift;

   print "Looking for $serviceAttr = $serviceValue\n";

   my @services = qx($appNoAF --sf "$serviceAttr = '$serviceValue'");

   $tests++;

   #Count the number of rows beginning with $serviceAttr that don't
   #contain $serviceValue
   if ( grep(!/$serviceValue/i, grep(/^Service $serviceAttr/i, @services)) == 0 ) {
   } else {
      print "Fail, other values were returned\n";

      #Get every $serviceAttr that isn't $serviceValue
      my @tmpArr = grep(!/$serviceValue/i, grep(/^Service $serviceAttr/i, @services));

      #Get unique values
      my %tmpHash;
      my @uniqueArr = grep(!$tmpHash{$_}++, @tmpArr);

      foreach ( @uniqueArr ) {
         print "$_\n";
      }
      $errors++;
   }

   my $numServices = grep(/^Service $serviceAttr/i, @services);
   print "Total number of Services with " .
         "$serviceAttr = $serviceValue = $numServices\n";

   return $numServices;
}

sub GetCountForNotServiceType($) {
   my $serviceType = shift;

   print "Looking for Service Type NOT = $serviceType\n";
   print "Check that this type is not returned\n";

   my @services =
      qx($appNoAF --sf "TYPE <> '$serviceType'" | grep -i "$searchType");

   $tests++;
   if ( grep(/$serviceType/i, @services) == 0 ) {
      print "Good, didn't find $serviceType\n";
   } else  {
      print "Fail, TYPE $serviceType was returned\n";
      $errors++;
   }

   my $numServices = grep(/$searchType/i, @services);
   print "Total number of Services with TYPE NOT = " .
         "$serviceType = $numServices\n";

   return $numServices;
}

#Takes an array of elements in the form ATTRIBUTE, VALUE, ATTRIBUTE, VALUE...
sub GetCountForServices(@) {
   my @serviceArr = @_;

   my $serviceString;

   #Check that we have an even number of elements in our array
   if ( ((@serviceArr % 2) != 0) ||
        (@serviceArr == 0) )
   {
      print "GetCountForServices(): array missing values\n";
      print "Using empty string\n";
      $serviceString = '';
   }

   else
   {
      print "Looking for Services :- \n";
      my $attr;
      my $value;

      $attr = shift(@serviceArr);
      $value = shift(@serviceArr);
      print "$attr = $value";
      $serviceString = "$attr = '$value' ";

      while ( @serviceArr )
      {
         $attr = shift(@serviceArr);
         $value = shift(@serviceArr);
         print " OR $attr = $value";
         $serviceString .= "OR $attr = '$value' ";
      }
      print "\n";
   }

   my @services = qx($appNoAF --sf "$serviceString");

   my $numServices = grep(/$searchType/i, @services);

   return $numServices;
}

sub GetCountForVo($) {
   my $voValue = shift;

   print "Looking for VO = $voValue\n";

   my $voFilter = "VO = '$voValue'";

   my @services = qx($app --af "$voFilter");

   $tests++;

   my $numServices = grep(/$searchService/, @services);

   return $numServices;
}

sub GetCountForVoIn(@)
{
   my @voValues = @_;

   print "Looking for VO in " . join(', ', @voValues) . "\n";

   my $voFilter = 'VO IN (' . join(', ', map("'$_'", @voValues )) . ')';

   my @services = qx($app --af "$voFilter");

   $tests++;

   my $numServices = grep(/$searchService/, @services);

   print "Total number of Services with VO IN " . join(', ', @voValues) .
         " = $numServices\n";

   return $numServices;
}

sub ConfirmValueNotPresentForVo($@)
{
   my $voValue = shift;
   my @services = @_;

   my $count = grep(/VO :[\s]*$voValue[\s]*$/i, @services);

   if ( $count > 0 )
   {
      print "Failed, entries contained VO = '$voValue'\n";
      $errors++;
   }
   return $count;
}

sub PassFail($$) {
   my $expected = shift;
   my $got = shift;

   if ( $expected == $got ) {
      print "Passed: Got $got\n";
      return 1;
   } else {
      print "Failed: Expected $expected got $got\n";
      $errors++;
      exit 1;
      return 0;
   }
}
