#!/usr/bin/perl -w
#
# Copyright (c) Members of the EGEE Collaboration. 2009-2010.
# See http://www.eu-egee.org/partners/ for details on the copyright
# holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
use strict;

sub CheckInfoServiceUrl($$);
sub RestartBDII();
sub RunTests();

my $TEST_BDII=shift;
if (! $TEST_BDII) {
    print "Please specify the name of the machine set up as a test BDII\n";
    exit 1;
}

my $BDII_WAIT=shift;
if (! $BDII_WAIT) {
    print "Please specify the number of seconds to wait for the BDII to reinitialize itself\n";
    exit 1;
}

my $BDII_URL="ldap://lcg-bdii.gridpp.ac.uk:2170";
my $LCG_GFAL_INFOSYS=$TEST_BDII.":2170";
my $LCG_GFAL_INFOSYS_URL="ldap://".$LCG_GFAL_INFOSYS;
my $CERN_BDII="ldap://lcg-bdii.cern.ch:2170";
my $cmd;

if (-e "default.ldif.gz") {
    print "Unpacking default.ldif\n";$cmd = "ssh root@".$TEST_BDII." service bdii restart";
    $cmd = "gunzip -f default.ldif.gz";
    if (system($cmd) != 0) {exit 1};
}

print "Set up fixed BDII configuration from CVS\n";
$cmd = "scp default.ldif root@".$TEST_BDII.":/opt/glite/etc/gip/ldif/default.ldif";
if (system($cmd) != 0) {exit 1};
RestartBDII();


# Clean up the environment
delete $ENV{BDII_URL};
delete $ENV{LCG_GFAL_INFOSYS};

CheckInfoServiceUrl("", $CERN_BDII);
CheckInfoServiceUrl("BDII_URL=".$BDII_URL, $BDII_URL);
CheckInfoServiceUrl("BDII_URL=".$BDII_URL." LCG_GFAL_INFOSYS=".$LCG_GFAL_INFOSYS , $BDII_URL);
CheckInfoServiceUrl("LCG_GFAL_INFOSYS=".$LCG_GFAL_INFOSYS, $LCG_GFAL_INFOSYS_URL);

# Set BDII_DIR env variable
$ENV{BDII_URL} = $LCG_GFAL_INFOSYS_URL;

RunTests();

print "Set up fixed BDII configuation from CERN top level BDII\n";
$cmd = "ldapsearch -x -H ".$CERN_BDII." -b mds-vo-name=local,o=grid -LLL > default.ldif.live";
if (system($cmd) != 0) {exit 1};
if (system("./cleanldif.py") != 0) {exit 1};
$cmd = "scp default.ldif.clean root@".$TEST_BDII.":/opt/glite/etc/gip/ldif/default.ldif";
if (system($cmd) != 0) {exit 1};
RestartBDII();

RunTests();


sub RunTests() {
    foreach ("./consistency.pl", "./bad.pl", "./good.pl") {
	print "\nNow running ", $_, "\n";
	if (system($_) != 0) {exit 1};
    }
}

sub RestartBDII() {
    my $cmd = "ssh root@".$TEST_BDII." service bdii restart";
    if (system($cmd) != 0) {exit 1};
    print "Now sleep for ", $BDII_WAIT, " seconds\n";
    sleep($BDII_WAIT);
    my $output = qx(saga-sd --af "" --sa -n 1 --url $LCG_GFAL_INFOSYS_URL ) or die();
    print $output;
}

sub CheckInfoServiceUrl($$) {
    my $env = shift;
    my $url = (shift)."/";
    print "With env: ", $env, " expect url ", $url, "\n";

    my $output = qx($env saga-sd --af "" -n 1 --sa | grep InformationServiceUrl) or die();
    my $ele3 = (split(/ /, $output))[3];
    $ele3 =~ s/\s+$//;
    if ($ele3 ne $url) {
	print "Failed as got: ", $ele3, "\n";
	exit 1;
    }
    
    $output = qx($env saga-sd --af "" -n 1 --sa --url $LCG_GFAL_INFOSYS_URL | grep InformationServiceUrl) or die();
    $ele3 = (split(/ /, $output))[3];
    $ele3 =~ s/\s+$//;
    $url =  $LCG_GFAL_INFOSYS_URL."/";
    if ($ele3 ne $url) {
	print "Failed as got: ", $ele3, " with -url override\n";
	exit 1;
    }
}






